@@ -1,8 +1,8 @@
 package POEIKC;
 
 use strict;
-use 5.8.1;
-our $VERSION = '0.03';
+use 5.008_001;
+our $VERSION = '0.04';
 
 1;
 __END__
