@@ -1,7 +1,7 @@
 package POEIKC::Daemon::Utility;
 
 use strict;
-use v5.8.1;
+use 5.008_001;
 
 use warnings;
 use Data::Dumper ();
@@ -46,11 +46,124 @@ sub shutdown {
 		$args{poe}, $args{rsvp}, $args{from}, $args{args} );
 	my ($alias, $event_ary) = ($args{alias}, $args{event});
 
-	$poe->kernel->yield('_shutdown');
+	$poe->kernel->yield('shutdown');
 	return sprintf("%s PID:%s ... stopped!! (%s)\n", $0, $$, scalar(localtime));
 }
 
 ### loop, relay, chain vvvvvvvvvvvvvvvvvvvvvvvvv
+package POEIKC::Daemon::State;
+
+use base qw/Class::Accessor::Fast/;
+
+__PACKAGE__->mk_accessors(qw/event_name next_mthod destination module method module_delay/);
+
+sub new {
+	my $class = shift ;
+	my %args = @_;
+	my $self = $class->SUPER::new(\%args);
+	$self->{limit} = $args{limit};
+	return $self ;
+}
+
+
+package POEIKC::Daemon::Utility;
+
+
+sub stop{
+	my $self = shift;
+	my %args = @_;
+	my ($poe, $rsvp, $from, $args) = (
+		$args{poe}, $args{rsvp}, $args{from}, $args{args} );
+	my $kernel = $poe->kernel;
+	my ($something, $method, @args) = @{$args} if ref $args eq 'ARRAY';
+	$something ||= $args{something};
+
+	$self->state_list->{$something} or return;
+
+	if (ref $self->state_list->{$something} eq 'HASH') {
+		my $event_name = $self->state_list->{$something}->event_name;
+		$DEBUG and _DEBUG_log( $event_name, $something);
+		$kernel->state( $event_name );
+		if ($method) {
+			my $destination = $self->state_list->{$something}->destination;
+			my $module = $self->state_list->{$something}->module;
+			$DEBUG and _DEBUG_log($self->state_list->{$something}, $event_name, $something, $destination, $module, $method, @args);
+			$DEBUG and _DEBUG_log($destination, $module, $method, @args);
+			return $self->execute(poe=>$poe, from=>$destination, module=>$module, method=>$method, args=>\@args);
+		}
+		delete $self->state_list->{$something};
+		return $event_name;
+	}
+}
+
+
+sub loop {
+	my $self = shift;
+	my %args = @_;
+	my ($poe, $rsvp, $from, $args) = (
+		$args{poe}, $args{rsvp}, $args{from}, $args{args} );
+	my $kernel = $poe->kernel;
+
+	$DEBUG and _DEBUG_log($args);
+
+	my $limit = shift @{$args} if ($args->[0] and $args->[0] =~ /^\d+$/);
+
+	my $func_full_name = $args->[0] || return;
+	$DEBUG and _DEBUG_log("func_full_name=>"=>$func_full_name);
+	$DEBUG and _DEBUG_log($args);
+
+	my $destination;
+	($destination, $args) = $self->_distinguish(poe=>$poe, args=>$args);
+	my $module = shift @{$args};
+	my $method = shift @{$args};
+	$DEBUG and _DEBUG_log("module=>"=>$module);
+
+	$self->use(module=>$module) or return $@;
+
+	my $module_delay = '\$'."${module}::delay";
+	$module_delay = eval $module_delay;
+
+	my $event_name = join "_" => $module =~ /(\w+)/, $method, 'loop';
+	$DEBUG and _DEBUG_log($event_name);
+
+	if ($func_full_name) {
+		$self->state_list->{$func_full_name} =
+			POEIKC::Daemon::State->new(
+				event_name	=>$event_name,
+				next_mthod	=>'',
+				limit	=>$limit,
+				destination	=>$destination,
+				module	=>$module,
+				method	=>$method,
+				module_delay	=>$module_delay
+			);
+		$kernel->state( $event_name , sub {
+				my @args = @_[POE::Session::ARG0() ..$#_];
+				$DEBUG and _DEBUG_log(@args);
+				if ($limit and not $self->state_list->{$func_full_name}->{limit}) {
+					$self->stop(poe=>$poe, something=>$func_full_name, );
+					return;
+				}
+				my @re_args = $self->execute(
+					poe=>$poe, from=>$destination, module=>$module, method=>$method, args=>\@args
+				);
+
+				my $delay = ${$self->state_list->{$func_full_name}->module_delay};
+				$DEBUG and _DEBUG_log($delay);
+				$delay ?
+				$kernel->delay($event_name => $delay, @re_args) :
+				$kernel->yield($event_name, @re_args);
+
+				$self->state_list->{$func_full_name}->{limit}-- if ($limit);
+			}
+		);
+		$kernel->yield($event_name, @{$args});
+		return $event_name;
+	}else{
+		return;
+	}
+}
+
 
 sub relay #{}
 {
@@ -71,16 +184,21 @@ sub relay #{}
 
 	$self->use(module=>$module) or return $@;
 
+	my $module_delay = '\$'."${module}::delay";
+	$module_delay = eval $module_delay;
+
 	my $event_name = join "_" => $module =~ /(\w+)/, $method, 'relay';
 
 	$DEBUG and _DEBUG_log($event_name, $something);
 
 	if ($something) {
-		$self->state_list->{$something} = {
-			event_name=>$event_name,
-			next_mthod=>'',
-			destination=>[$destination, $module, $method],
-		};
+		$self->state_list->{$something} =
+			POEIKC::Daemon::State->new(
+				event_name=>$event_name,
+				next_mthod=>'',
+				destination=>[$destination, $module, $method],
+				module_delay	=>$module_delay
+			);
 		$kernel->state( $event_name , sub {
 				my @args = @_[POE::Session::ARG0() ..$#_];
 				$DEBUG and _DEBUG_log(\@args);
@@ -90,11 +208,11 @@ sub relay #{}
 				$DEBUG and _DEBUG_log(@re_args);
 				my $delay = shift @re_args if ($re_args[0] and not(ref $re_args[0]) and $re_args[0] =~ /^\d+$/);
 				my $next_mthod = $re_args[0] if ($re_args[0] and not(ref $re_args[0]) and $re_args[0] =~ /^\w+/);
-				
-				(not $next_mthod) ? 
-					$self->stop(poe=>$poe, something=>$something,) : 
-				$delay ? 
-					$kernel->delay($event_name => $delay, @re_args ) : 
+
+				(not $next_mthod) ?
+					$self->stop(poe=>$poe, something=>$something,) :
+				$delay ?
+					$kernel->delay($event_name => $delay, @re_args ) :
 					$kernel->yield($event_name, @re_args ) ;
 			}
 		);
@@ -126,17 +244,22 @@ sub chain #{}
 
 	$self->use(module=>$module) or return $@;
 
+	my $module_delay = '\$'."${module}::delay";
+	$module_delay = eval $module_delay;
+
 	my $event_name = join "_" => $module =~ /(\w+)/, $method, 'chain';
 
 	$DEBUG and _DEBUG_log($event_name, $something);
 
 	if ($something) {
-		$self->state_list->{$something} = {
-			event_name=>$event_name,
-			next_mthod=>\@next_mthod,
-			pointer=>0,
-			destination=>[$destination, $module, $method],
-		};
+		$self->state_list->{$something} =
+			POEIKC::Daemon::State->new(
+				event_name=>$event_name,
+				next_mthod=>\@next_mthod,
+				pointer=>0,
+				destination=>[$destination, $module, $method],
+				module_delay	=>$module_delay
+			);
 		$kernel->state( $event_name , sub {
 				my @args = @_[POE::Session::ARG0() ..$#_];
 				$DEBUG and _DEBUG_log(\@args);
@@ -144,9 +267,9 @@ sub chain #{}
 				my $method = $self->state_list->{$something}->{next_mthod}->[$pointer];
 				my @re_args = $self->execute(poe=>$poe, from=>$destination, module=>$module, method=>$method, args=>\@args);
 				$self->state_list->{$something}->{pointer}++;
-				($pointer >= $#{$self->state_list->{$something}->{next_mthod}}) 
+				($pointer >= $#{$self->state_list->{$something}->{next_mthod}})
 					?
-					$self->stop(poe=>$poe, something=>$something,) : 
+					$self->stop(poe=>$poe, something=>$something,) :
 					$kernel->yield($event_name, @re_args ) ;
 			}
 		);
@@ -157,91 +280,7 @@ sub chain #{}
 	}
 }
 
-# -U=loop #delay #limit  module::method , args ..);
-# -U=loop_stop   module::method );
 
-sub loop { 
-	my $self = shift;
-	my %args = @_;
-	my ($poe, $rsvp, $from, $args) = (
-		$args{poe}, $args{rsvp}, $args{from}, $args{args} );
-	my $kernel = $poe->kernel;
-
-	$DEBUG and _DEBUG_log($args);
-
-	my $limit = shift @{$args} if ($args->[0] and $args->[0] =~ /^\d+$/);
-
-	my $something = $args->[0] || return;
-	$DEBUG and _DEBUG_log($something);
-	$DEBUG and _DEBUG_log($args);
-
-	my $destination;
-	($destination, $args) = $self->_distinguish(poe=>$poe, args=>$args);
-	my $module = shift @{$args};
-	my $method = shift @{$args};
-
-	$self->use(module=>$module) or return $@;
-
-	my $event_name = join "_" => $module =~ /(\w+)/, $method, 'loop';
-	$DEBUG and _DEBUG_log($event_name);
-
-	if ($something) {
-		$self->state_list->{$something} = {
-			event_name	=>$event_name,
-			next_mthod	=>'',
-			limit	=>$limit,
-			destination	=>$destination, 
-			module	=>$module,
-			method	=>$method,
-		};
-		$kernel->state( $event_name , sub {
-				my @args = @_[POE::Session::ARG0() ..$#_];
-				$DEBUG and _DEBUG_log(@args);
-				if ($limit and not $self->state_list->{$something}->{limit}) {
-					$self->stop(poe=>$poe, something=>$something, );
-					return;
-				}
-				my @re_args = $self->execute(
-					poe=>$poe, from=>$destination, module=>$module, method=>$method, args=>\@args
-				);
-				$kernel->yield($event_name, @re_args);
-				## $kernel->delay($event_name => 0.05, @re_args);
-				$self->state_list->{$something}->{limit}-- if ($limit);
-			}
-		);
-		$kernel->yield($event_name, @{$args});
-		return $event_name;
-	}else{
-		return;
-	}
-}
-
-sub stop{
-	my $self = shift;
-	my %args = @_;
-	my ($poe, $rsvp, $from, $args) = (
-		$args{poe}, $args{rsvp}, $args{from}, $args{args} );
-	my $kernel = $poe->kernel;
-	my ($something, $method, @args) = @{$args} if ref $args eq 'ARRAY';
-	$something ||= $args{something};
-
-	$self->state_list->{$something} or return;
-
-	if (ref $self->state_list->{$something} eq 'HASH') {
-		my $event_name = $self->state_list->{$something}->{event_name};
-		$DEBUG and _DEBUG_log( $event_name, $something);
-		$kernel->state( $event_name );
-		if ($method) {
-			my $destination = $self->state_list->{$something}->{destination};
-			my $module = $self->state_list->{$something}->{module};
-			$DEBUG and _DEBUG_log($self->state_list->{$something}, $event_name, $something, $destination, $module, $method, @args);
-			$DEBUG and _DEBUG_log($destination, $module, $method, @args);
-			return $self->execute(poe=>$poe, from=>$destination, module=>$module, method=>$method, args=>\@args);
-		}
-		delete $self->state_list->{$something};
-		return $event_name;
-	}
-}
 
 ### exec vvvvvvvvvvvvvvvvvvvvvvvvv
 
@@ -254,16 +293,16 @@ sub execute {
 	my $kernel = $poe->kernel if $poe;
 	#_DEBUG_log(wantarray);
 	for ($from) {
-		/^method/	and return eval { 
+		/^method/	and return eval {
 				$DEBUG and _DEBUG_log("$module->$method( @{$args} )");
 				$module->$method( @{$args} )
 				} ;
-		/^event/	and return eval { 
+		/^event/	and return eval {
 				local $! = undef;
 				$DEBUG and _DEBUG_log("call( $module => $method, @{$args} )");
 				$kernel->call( $module => $method, @{$args} )or return $!
 				} ;
-		/^function/	and return eval { 
+		/^function/	and return eval {
 				no strict 'refs';
 				my $code = "${module}::$method";
 				$DEBUG and _DEBUG_log($code);
@@ -275,7 +314,7 @@ sub execute {
 	}
 	return {poeikcd_error=>
 				'It is not discriminable. '.
-				q{"ModuleName::functionName" or  "ClassName->methodName" or "AliasName eventName"} 
+				q{"ModuleName::functionName" or  "ClassName->methodName" or "AliasName eventName"}
 			}
 }
 
@@ -492,7 +531,7 @@ sub publish_IKC {
 
 
 
-### 
+###
 
 sub _distinguish {
 	my $self = shift;
@@ -535,7 +574,7 @@ sub _distinguish {
 		};
 	}
 	$DEBUG and _DEBUG_log();
-	return 
+	return
 }
 
 ### DEBUG vvvvvvvvvvvvvvvvvvvvvvvvv
@@ -562,8 +601,8 @@ sub _DEBUG_log {
 	my @data = @_;
 	print(
 		$log_header, (join "\t" => map {
-			ref($_) ? Data::Dumper::Dumper($_) : 
-			defined $_ ? $_ : "`'" ; 
+			ref($_) ? Data::Dumper::Dumper($_) :
+			defined $_ ? $_ : "`'" ;
 		} @data ),"\n"
 	);
 }
@@ -597,7 +636,7 @@ POEIKC::Daemon::Utility - Utility for POEIKC::Daemon
 
 =head1 SYNOPSIS
 
-	$ret = $ikc->post_respond('POEIKCd/method_respond' => 
+	$ret = $ikc->post_respond('POEIKCd/method_respond' =>
 		['POEIKC::Daemon::Utility'=> $method_name, $args ..] );
 	print Dumper $ret;
 
@@ -607,7 +646,7 @@ POEIKC::Daemon::Utility is Utility for POEIKC::Daemon
 
 =head1 AUTHOR
 
-Yuji Suzuki E<lt>yuji.suzuki.perl@gmail.comE<gt>
+Yuji Suzuki E<lt>yujisuzuki@mail.arbolbell.jpE<gt>
 
 =head1 LICENSE
 
